@@ -1,4 +1,7 @@
+use std::io;
+
 use crate::common::{HARDENED_ENUM_VALUE_0, HARDENED_ENUM_VALUE_1, HARDENED_ENUM_VALUE_2};
+use crate::exec::RunOptions;
 use crate::sudo::{SudoListOptions, SudoRunOptions, SudoValidateOptions};
 use crate::sudoers::Sudoers;
 use crate::system::{Group, Hostname, Process, User};
@@ -18,6 +21,8 @@ pub struct Context {
     pub target_user: User,
     pub target_group: Group,
     pub stdin: bool,
+    pub bell: bool,
+    pub prompt: Option<String>,
     pub non_interactive: bool,
     pub use_session_records: bool,
     // system
@@ -84,6 +89,8 @@ impl Context {
             launch,
             chdir: sudo_options.chdir,
             stdin: sudo_options.stdin,
+            bell: sudo_options.bell,
+            prompt: sudo_options.prompt,
             non_interactive: sudo_options.non_interactive,
             process: Process::new(),
             use_pty: true,
@@ -107,6 +114,8 @@ impl Context {
             launch: Default::default(),
             chdir: None,
             stdin: sudo_options.stdin,
+            bell: sudo_options.bell,
+            prompt: sudo_options.prompt,
             non_interactive: sudo_options.non_interactive,
             process: Process::new(),
             use_pty: true,
@@ -150,12 +159,32 @@ impl Context {
             launch: Default::default(),
             chdir: None,
             stdin: sudo_options.stdin,
+            bell: sudo_options.bell,
+            prompt: sudo_options.prompt,
             non_interactive: sudo_options.non_interactive,
             process: Process::new(),
             use_pty: true,
             password_feedback: false,
         })
     }
+
+    pub(crate) fn try_as_run_options(&self) -> io::Result<RunOptions<'_>> {
+        Ok(RunOptions {
+            command: if self.command.resolved {
+                &self.command.command
+            } else {
+                return Err(io::ErrorKind::NotFound.into());
+            },
+            arguments: &self.command.arguments,
+            arg0: self.command.arg0.as_deref(),
+            chdir: self.chdir.as_deref(),
+            is_login: self.launch == LaunchType::Login,
+            user: &self.target_user,
+            group: &self.target_group,
+
+            use_pty: self.use_pty,
+        })
+    }
 }
 
 #[cfg(test)]
