@@ -1,5 +1,4 @@
 #![forbid(unsafe_code)]
-use std::{collections::HashMap, ffi::OsString};
 
 pub use command::CommandAndArguments;
 pub use context::Context;
@@ -15,8 +14,6 @@ mod path;
 pub mod resolve;
 mod string;
 
-pub type Environment = HashMap<OsString, OsString>;
-
 // Hardened enum values used for critical enums to mitigate attacks like Rowhammer.
 // See for example https://arxiv.org/pdf/2309.02545.pdf
 // The values are copied from https://github.com/sudo-project/sudo/commit/7873f8334c8d31031f8cfa83bd97ac6029309e4f#diff-b8ac7ab4c3c4a75aed0bb5f7c5fd38b9ea6c81b7557f775e46c6f8aa115e02cd
