@@ -35,6 +35,7 @@ defaults! {
     pwfeedback                = false
     env_editor                = true
     rootpw                    = false
+    targetpw                  = false
 
     passwd_tries              = 3 [0..=1000]
 
@@ -61,10 +62,18 @@ defaults! {
 /// A custom parser to parse seconds as fractional "minutes", the format used by
 /// passwd_timeout and timestamp_timeout.
 fn fractional_minutes(input: &str) -> Option<i64> {
-    if input.contains('.') {
-        Some((input.parse::<f64>().ok()? * 60.0).floor() as i64)
+    if let Some((integral, fractional)) = input.split_once('.') {
+        // - 'input' is maximally 18 characters, making fractional.len() at most 17;
+        //   1e17 < 2**63, so the definition of 'shift' will not overflow.
+        // - for the same reason, if both parses in the definition of 'seconds' succeed,
+        //   we will have constructed an integer < 1e17.
+        //-  1e17 * 60 = 6e18 < 9e18 < 2**63, so the final line also will not overflow
+        let shift = 10i64.pow(fractional.len().try_into().ok()?);
+        let seconds = integral.parse::<i64>().ok()? * shift + fractional.parse::<i64>().ok()?;
+
+        Some(seconds * 60 / shift)
     } else {
-        Some(input.parse::<i64>().ok()? * 60)
+        input.parse::<i64>().ok()?.checked_mul(60)
     }
 }
 
@@ -122,6 +131,10 @@ mod test {
             panic!()
         };
         f("any").unwrap()(&mut def);
+        let SettingKind::Integer(f) = set("timestamp_timeout").unwrap() else {
+            panic!()
+        };
+        f("25.25").unwrap()(&mut def);
         assert_eq! { def.always_query_group_plugin, false };
         assert_eq! { def.always_set_home, false };
         assert_eq! { def.env_reset, true };
@@ -131,6 +144,7 @@ mod test {
         assert_eq! { def.visiblepw, false };
         assert_eq! { def.env_editor, true };
         assert_eq! { def.passwd_tries, 5 };
+        assert_eq! { def.timestamp_timeout, 25*60 + 60/4 };
         assert_eq! { def.secure_path, Some("/bin".into()) };
         assert! { def.env_check.is_empty() };
         assert_eq! { def.verifypw, enums::verifypw::any };
