@@ -24,10 +24,10 @@ pub mod sys;
 #[cfg(target_os = "freebsd")]
 const PAM_DATA_SILENT: std::ffi::c_int = 0;
 
-pub use converse::{CLIConverser, Converser};
+pub use converse::CLIConverser;
 
-pub struct PamContext<C: Converser> {
-    data_ptr: *mut ConverserData<C>,
+pub struct PamContext {
+    data_ptr: *mut ConverserData<CLIConverser>,
     pamh: *mut pam_handle_t,
     silent: bool,
     allow_null_auth_token: bool,
@@ -35,7 +35,7 @@ pub struct PamContext<C: Converser> {
     session_started: bool,
 }
 
-impl PamContext<CLIConverser> {
+impl PamContext {
     /// Build the PamContext with the CLI conversation function.
     ///
     /// The target user is optional and may also be set after the context was
@@ -47,27 +47,18 @@ impl PamContext<CLIConverser> {
         converser_name: &str,
         service_name: &str,
         use_stdin: bool,
+        bell: bool,
         no_interact: bool,
         password_feedback: bool,
         target_user: Option<&str>,
-    ) -> PamResult<PamContext<CLIConverser>> {
+    ) -> PamResult<PamContext> {
         let converser = CLIConverser {
+            bell,
             name: converser_name.to_owned(),
             use_stdin,
-            no_interact,
             password_feedback,
         };
 
-        Self::new(converser, service_name, target_user)
-    }
-}
-
-impl<C: Converser> PamContext<C> {
-    fn new(
-        converser: C,
-        service_name: &str,
-        target_user: Option<&str>,
-    ) -> PamResult<PamContext<C>> {
         let c_service_name = CString::new(service_name)?;
         let c_user = target_user.map(CString::new).transpose()?;
         let c_user_ptr = match c_user {
@@ -78,6 +69,12 @@ impl<C: Converser> PamContext<C> {
         // this will be de-allocated explicitly in this type's drop method
         let data_ptr = Box::into_raw(Box::new(ConverserData {
             converser,
+            converser_name: converser_name.to_owned(),
+            no_interact,
+            auth_prompt: std::env::var("SUDO_PROMPT")
+                .ok()
+                .filter(|s| !s.is_empty())
+                .or(Some("authenticate".to_owned())),
             panicked: false,
         }));
 
@@ -89,7 +86,7 @@ impl<C: Converser> PamContext<C> {
                 c_service_name.as_ptr(),
                 c_user_ptr,
                 &pam_conv {
-                    conv: Some(converse::converse::<C>),
+                    conv: Some(converse::converse::<CLIConverser>),
                     appdata_ptr: data_ptr as *mut libc::c_void,
                 },
                 &mut pamh,
@@ -110,6 +107,13 @@ impl<C: Converser> PamContext<C> {
         })
     }
 
+    pub fn set_auth_prompt(&mut self, prompt: Option<String>) {
+        // SAFETY: self.data_ptr was created by Box::into_raw
+        unsafe {
+            (*self.data_ptr).auth_prompt = prompt;
+        }
+    }
+
     /// Set whether output of pam calls should be silent or not, by default
     /// PAM calls are not silent.
     pub fn mark_silent(&mut self, silent: bool) {
@@ -351,7 +355,7 @@ impl<C: Converser> PamContext<C> {
     }
 }
 
-impl<C: Converser> Drop for PamContext<C> {
+impl Drop for PamContext {
     fn drop(&mut self) {
         // data_ptr's pointee is de-allocated in this scope
         // SAFETY: self.data_ptr was created by Box::into_raw
