@@ -1,9 +1,9 @@
 #![forbid(unsafe_code)]
 
 use crate::common::error::Error;
-use crate::exec::{ExecOutput, ExitReason, RunOptions};
+use crate::exec::ExitReason;
 use crate::log::user_warn;
-use crate::pam::{CLIConverser, PamContext, PamError, PamErrorType};
+use crate::pam::{PamContext, PamError, PamErrorType};
 use crate::system::term::current_tty_name;
 
 use std::{env, process};
@@ -21,11 +21,7 @@ mod help;
 const DEFAULT_USER: &str = "root";
 const VERSION: &str = env!("CARGO_PKG_VERSION");
 
-fn authenticate(
-    requesting_user: &str,
-    user: &str,
-    login: bool,
-) -> Result<PamContext<CLIConverser>, Error> {
+fn authenticate(requesting_user: &str, user: &str, login: bool) -> Result<PamContext, Error> {
     // FIXME make it configurable by the packager
     let context = if login && cfg!(target_os = "linux") {
         "su-l"
@@ -33,7 +29,7 @@ fn authenticate(
         "su"
     };
     let use_stdin = true;
-    let mut pam = PamContext::new_cli("su", context, use_stdin, false, false, Some(user))?;
+    let mut pam = PamContext::new_cli("su", context, use_stdin, false, false, false, Some(user))?;
     pam.set_requesting_user(requesting_user)?;
 
     // attempt to set the TTY this session is communicating on
@@ -88,10 +84,10 @@ fn run(options: SuRunOptions) -> Result<(), Error> {
     let context = SuContext::from_env(options)?;
 
     // authenticate the target user
-    let mut pam: PamContext<CLIConverser> = authenticate(
+    let mut pam: PamContext = authenticate(
         &context.requesting_user.name,
-        &context.user().name,
-        context.is_login(),
+        &context.user.name,
+        context.options.login,
     )?;
 
     // su in all cases uses PAM (pam_getenvlist(3)) to do the
@@ -104,17 +100,11 @@ fn run(options: SuRunOptions) -> Result<(), Error> {
     let pid = context.process.pid;
 
     // run command and return corresponding exit code
-    let ExecOutput {
-        command_exit_reason,
-        restore_signal_handlers,
-    } = crate::exec::run_command(&context, environment)?;
+    let command_exit_reason = crate::exec::run_command(context.as_run_options(), environment);
 
     pam.close_session();
 
-    // Run any clean-up code before this line.
-    restore_signal_handlers();
-
-    match command_exit_reason {
+    match command_exit_reason? {
         ExitReason::Code(code) => process::exit(code),
         ExitReason::Signal(signal) => {
             crate::system::kill(pid, signal)?;
