@@ -167,12 +167,16 @@ impl TryFrom<SudoOptions> for SudoResetTimestampOptions {
 
 // sudo -v [-ABkNnS] [-g group] [-h host] [-p prompt] [-u user]
 pub struct SudoValidateOptions {
+    // -B
+    pub bell: bool,
     // -k
     pub reset_timestamp: bool,
     // -n
     pub non_interactive: bool,
     // -S
     pub stdin: bool,
+    // -p
+    pub prompt: Option<String>,
     // -g
     pub group: Option<SudoString>,
     // -u
@@ -187,18 +191,26 @@ impl TryFrom<SudoOptions> for SudoValidateOptions {
         let validate = mem::take(&mut opts.validate);
         debug_assert!(validate);
 
+        let bell = mem::take(&mut opts.bell);
         let reset_timestamp = mem::take(&mut opts.reset_timestamp);
         let non_interactive = mem::take(&mut opts.non_interactive);
         let stdin = mem::take(&mut opts.stdin);
+        let prompt = mem::take(&mut opts.prompt);
         let group = mem::take(&mut opts.group);
         let user = mem::take(&mut opts.user);
 
+        if bell && stdin {
+            return Err("--bell conflicts with --stdin".into());
+        }
+
         reject_all("--validate", opts)?;
 
         Ok(Self {
+            bell,
             reset_timestamp,
             non_interactive,
             stdin,
+            prompt,
             group,
             user,
         })
@@ -208,12 +220,16 @@ impl TryFrom<SudoOptions> for SudoValidateOptions {
 // sudo -e [-ABkNnS] [-r role] [-t type] [-C num] [-D directory] [-g group] [-h host] [-p prompt] [-R directory] [-T timeout] [-u user] file ...
 #[allow(dead_code)]
 pub struct SudoEditOptions {
+    // -B
+    pub bell: bool,
     // -k
     pub reset_timestamp: bool,
     // -n
     pub non_interactive: bool,
     // -S
     pub stdin: bool,
+    // -p
+    pub prompt: Option<String>,
     // -D
     pub chdir: Option<SudoPath>,
     // -g
@@ -231,14 +247,20 @@ impl TryFrom<SudoOptions> for SudoEditOptions {
         let edit = mem::take(&mut opts.edit);
         debug_assert!(edit);
 
+        let bell = mem::take(&mut opts.bell);
         let reset_timestamp = mem::take(&mut opts.reset_timestamp);
         let non_interactive = mem::take(&mut opts.non_interactive);
         let stdin = mem::take(&mut opts.stdin);
+        let prompt = mem::take(&mut opts.prompt);
         let chdir = mem::take(&mut opts.chdir);
         let group = mem::take(&mut opts.group);
         let user = mem::take(&mut opts.user);
         let positional_args = mem::take(&mut opts.positional_args);
 
+        if bell && stdin {
+            return Err("--bell conflicts with --stdin".into());
+        }
+
         reject_all("--edit", opts)?;
 
         if positional_args.is_empty() {
@@ -246,9 +268,11 @@ impl TryFrom<SudoOptions> for SudoEditOptions {
         }
 
         Ok(Self {
+            bell,
             reset_timestamp,
             non_interactive,
             stdin,
+            prompt,
             chdir,
             group,
             user,
@@ -259,6 +283,8 @@ impl TryFrom<SudoOptions> for SudoEditOptions {
 
 // sudo -l [-ABkNnS] [-g group] [-h host] [-p prompt] [-U user] [-u user] [command [arg ...]]
 pub struct SudoListOptions {
+    // -B
+    pub bell: bool,
     // -l OR -l -l
     pub list: List,
 
@@ -268,6 +294,8 @@ pub struct SudoListOptions {
     pub non_interactive: bool,
     // -S
     pub stdin: bool,
+    // -p
+    pub prompt: Option<String>,
     // -g
     pub group: Option<SudoString>,
     // -U
@@ -282,15 +310,21 @@ impl TryFrom<SudoOptions> for SudoListOptions {
     type Error = String;
 
     fn try_from(mut opts: SudoOptions) -> Result<Self, Self::Error> {
+        let bell = mem::take(&mut opts.bell);
         let list = opts.list.take().unwrap();
         let reset_timestamp = mem::take(&mut opts.reset_timestamp);
         let non_interactive = mem::take(&mut opts.non_interactive);
         let stdin = mem::take(&mut opts.stdin);
+        let prompt = mem::take(&mut opts.prompt);
         let group = mem::take(&mut opts.group);
         let other_user = mem::take(&mut opts.other_user);
         let user = mem::take(&mut opts.user);
         let positional_args = mem::take(&mut opts.positional_args);
 
+        if bell && stdin {
+            return Err("--bell conflicts with --stdin".into());
+        }
+
         // when present, `-u` must be accompanied by a command
         let has_command = !positional_args.is_empty();
         let valid_user_flag = user.is_none() || has_command;
@@ -302,10 +336,12 @@ impl TryFrom<SudoOptions> for SudoListOptions {
         reject_all("--list", opts)?;
 
         Ok(Self {
+            bell,
             list,
             reset_timestamp,
             non_interactive,
             stdin,
+            prompt,
             group,
             other_user,
             user,
@@ -316,6 +352,8 @@ impl TryFrom<SudoOptions> for SudoListOptions {
 
 // sudo [-ABbEHnPS] [-C num] [-D directory] [-g group] [-h host] [-p prompt] [-R directory] [-T timeout] [-u user] [VAR=value] [-i | -s] [command [arg ...]]
 pub struct SudoRunOptions {
+    // -B
+    pub bell: bool,
     // -E
     pub preserve_env: PreserveEnv,
     // -k
@@ -324,6 +362,8 @@ pub struct SudoRunOptions {
     pub non_interactive: bool,
     // -S
     pub stdin: bool,
+    // -p
+    pub prompt: Option<String>,
     // -D
     pub chdir: Option<SudoPath>,
     // -g
@@ -343,10 +383,12 @@ impl TryFrom<SudoOptions> for SudoRunOptions {
     type Error = String;
 
     fn try_from(mut opts: SudoOptions) -> Result<Self, Self::Error> {
+        let bell = mem::take(&mut opts.bell);
         let preserve_env = mem::take(&mut opts.preserve_env);
         let reset_timestamp = mem::take(&mut opts.reset_timestamp);
         let non_interactive = mem::take(&mut opts.non_interactive);
         let stdin = mem::take(&mut opts.stdin);
+        let prompt = mem::take(&mut opts.prompt);
         let chdir = mem::take(&mut opts.chdir);
         let group = mem::take(&mut opts.group);
         let user = mem::take(&mut opts.user);
@@ -355,6 +397,10 @@ impl TryFrom<SudoOptions> for SudoRunOptions {
         let shell = mem::take(&mut opts.shell);
         let positional_args = mem::take(&mut opts.positional_args);
 
+        if bell && stdin {
+            return Err("--bell conflicts with --stdin".into());
+        }
+
         let context = match (login, shell, positional_args.is_empty()) {
             (true, false, _) => "--login",
             (false, true, _) => "--shell",
@@ -377,10 +423,12 @@ impl TryFrom<SudoOptions> for SudoRunOptions {
         reject_all(context, opts)?;
 
         Ok(Self {
+            bell,
             preserve_env,
             reset_timestamp,
             non_interactive,
             stdin,
+            prompt,
             chdir,
             group,
             user,
@@ -394,6 +442,8 @@ impl TryFrom<SudoOptions> for SudoRunOptions {
 
 #[derive(Default)]
 struct SudoOptions {
+    // -B
+    bell: bool,
     // -D
     chdir: Option<SudoPath>,
     // -g
@@ -410,6 +460,8 @@ struct SudoOptions {
     shell: bool,
     // -S
     stdin: bool,
+    // -p
+    prompt: Option<String>,
     // -u
     user: Option<SudoString>,
 
@@ -480,9 +532,16 @@ enum SudoArg {
 }
 
 impl SudoArg {
-    const TAKES_ARGUMENT_SHORT: &'static [char] = &['D', 'g', 'h', 'R', 'U', 'u'];
-    const TAKES_ARGUMENT: &'static [&'static str] =
-        &["chdir", "group", "host", "chroot", "other-user", "user"];
+    const TAKES_ARGUMENT_SHORT: &'static [char] = &['D', 'g', 'h', 'p', 'R', 'U', 'u'];
+    const TAKES_ARGUMENT: &'static [&'static str] = &[
+        "chdir",
+        "group",
+        "host",
+        "chroot",
+        "other-user",
+        "user",
+        "prompt",
+    ];
 
     /// argument assignments and shorthand options preprocessing
     fn normalize_arguments<I>(iter: I) -> Result<Vec<Self>, String>
@@ -603,6 +662,9 @@ impl SudoOptions {
         for arg in arg_iter {
             match arg {
                 SudoArg::Flag(flag) => match flag.as_str() {
+                    "-B" | "--bell" => {
+                        options.bell = true;
+                    }
                     "-E" | "--preserve-env" => {
                         options.preserve_env = PreserveEnv::Everything;
                     }
@@ -667,6 +729,9 @@ impl SudoOptions {
                     "-g" | "--group" => {
                         options.group = Some(SudoString::from_cli_string(value));
                     }
+                    "-p" | "--prompt" => {
+                        options.prompt = Some(value);
+                    }
                     "-U" | "--other-user" => {
                         options.other_user = Some(SudoString::from_cli_string(value));
                     }
@@ -751,6 +816,7 @@ fn reject_all(context: &str, opts: SudoOptions) -> Result<(), String> {
     }
 
     let SudoOptions {
+        bell,
         chdir,
         group,
         login,
@@ -759,6 +825,7 @@ fn reject_all(context: &str, opts: SudoOptions) -> Result<(), String> {
         preserve_env,
         shell,
         stdin,
+        prompt,
         user,
         env_var_list,
         edit,
@@ -772,6 +839,7 @@ fn reject_all(context: &str, opts: SudoOptions) -> Result<(), String> {
     } = opts;
 
     let flags = [
+        tuple!(bell),
         tuple!(chdir),
         tuple!(edit),
         tuple!(group),
@@ -785,6 +853,7 @@ fn reject_all(context: &str, opts: SudoOptions) -> Result<(), String> {
         tuple!(reset_timestamp),
         tuple!(shell),
         tuple!(stdin),
+        tuple!(prompt),
         tuple!(user),
         tuple!(validate),
         tuple!(version),
