@@ -77,20 +77,17 @@ fn parse_env_commands(input: &str) -> Vec<(&str, Environment)> {
         .collect()
 }
 
-fn create_test_context(sudo_options: &SudoRunOptions) -> Context {
+fn create_test_context(sudo_options: SudoRunOptions) -> Context {
     let path = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin".to_string();
-    let command =
-        CommandAndArguments::build_from_args(None, sudo_options.positional_args.clone(), &path);
+    let command = CommandAndArguments::build_from_args(None, sudo_options.positional_args, &path);
 
     let current_user = CurrentUser::fake(User {
         uid: UserId::new(1000),
         gid: GroupId::new(1000),
 
         name: "test".into(),
-        gecos: String::new(),
         home: "/home/test".into(),
         shell: "/bin/sh".into(),
-        passwd: String::new(),
         groups: vec![],
     });
 
@@ -103,10 +100,8 @@ fn create_test_context(sudo_options: &SudoRunOptions) -> Context {
         uid: UserId::ROOT,
         gid: GroupId::new(0),
         name: "root".into(),
-        gecos: String::new(),
         home: "/root".into(),
         shell: "/bin/bash".into(),
-        passwd: String::new(),
         groups: vec![],
     };
 
@@ -130,13 +125,15 @@ fn create_test_context(sudo_options: &SudoRunOptions) -> Context {
             root_group
         },
         launch: crate::common::context::LaunchType::Direct,
-        chdir: sudo_options.chdir.clone(),
+        chdir: sudo_options.chdir,
         stdin: sudo_options.stdin,
+        prompt: sudo_options.prompt,
         non_interactive: sudo_options.non_interactive,
         process: Process::new(),
         use_session_records: false,
         use_pty: true,
         password_feedback: false,
+        bell: false,
     }
 }
 
@@ -160,7 +157,7 @@ fn test_environment_variable_filtering() {
             .ok()
             .unwrap();
         let settings = crate::defaults::Settings::default();
-        let context = create_test_context(&options);
+        let context = create_test_context(options);
         let resulting_env = get_target_environment(
             initial_env.clone(),
             HashMap::new(),
