@@ -289,23 +289,41 @@ impl Parse for Meta<Identifier> {
 /// ```
 impl Parse for UserSpecifier {
     fn parse(stream: &mut CharStream) -> Parsed<Self> {
-        let userspec = if stream.eat_char('%') {
-            let ctor = if stream.eat_char(':') {
-                UserSpecifier::NonunixGroup
+        fn parse_user(stream: &mut CharStream) -> Parsed<UserSpecifier> {
+            let userspec = if stream.eat_char('%') {
+                let ctor = if stream.eat_char(':') {
+                    UserSpecifier::NonunixGroup
+                } else {
+                    UserSpecifier::Group
+                };
+                // in this case we must fail 'hard', since input has been consumed
+                ctor(expect_nonterminal(stream)?)
+            } else if stream.eat_char('+') {
+                // TODO Netgroups
+                unrecoverable!(stream, "netgroups are not supported yet");
             } else {
-                UserSpecifier::Group
+                // in this case we must fail 'softly', since no input has been consumed yet
+                UserSpecifier::User(try_nonterminal(stream)?)
             };
-            // in this case we must fail 'hard', since input has been consumed
-            ctor(expect_nonterminal(stream)?)
-        } else if stream.eat_char('+') {
-            // TODO Netgroups
-            unrecoverable!(stream, "netgroups are not supported yet");
-        } else {
-            // in this case we must fail 'softly', since no input has been consumed yet
-            UserSpecifier::User(try_nonterminal(stream)?)
-        };
 
-        make(userspec)
+            make(userspec)
+        }
+
+        // if we see a quote, first parse the quoted text as a token and then
+        // re-parse whatever we found inside; this is a lazy solution but it works
+        let begin_pos = stream.get_pos();
+        if stream.eat_char('"') {
+            let Unquoted(text, _): Unquoted<Username> = expect_nonterminal(stream)?;
+            let result = parse_user(&mut CharStream::new(text.chars()));
+            if result.is_err() {
+                unrecoverable!(pos = begin_pos, stream, "invalid user")
+            }
+            expect_syntax('"', stream)?;
+
+            result
+        } else {
+            parse_user(stream)
+        }
     }
 }
 
@@ -570,7 +588,7 @@ impl Parse for Sudo {
 fn parse_include(stream: &mut CharStream) -> Parsed<Sudo> {
     fn get_path(stream: &mut CharStream, key_pos: (usize, usize)) -> Parsed<(String, Span)> {
         let path = if stream.eat_char('"') {
-            let QuotedInclude(path) = expect_nonterminal(stream)?;
+            let QuotedIncludePath(path) = expect_nonterminal(stream)?;
             expect_syntax('"', stream)?;
             path
         } else {
@@ -704,10 +722,11 @@ impl Parse for defaults::SettingsModifier {
             if stream.eat_char('"') {
                 let mut result = Vec::new();
                 while let Some(EnvVar(name)) = try_nonterminal(stream)? {
-                    result.push(name);
                     if is_syntax('=', stream)? {
-                        let EnvVar(_) = expect_nonterminal(stream)?;
-                        unrecoverable!(stream, "values in environment variables not yet supported")
+                        let StringParameter(value) = expect_nonterminal(stream)?;
+                        result.push(name + "=" + &value);
+                    } else {
+                        result.push(name);
                     }
                     if result.len() > Identifier::LIMIT {
                         unrecoverable!(stream, "environment variable list too long")
@@ -721,8 +740,11 @@ impl Parse for defaults::SettingsModifier {
                 make(result)
             } else {
                 let EnvVar(name) = expect_nonterminal(stream)?;
-
-                make(vec![name])
+                if is_syntax('=', stream)? {
+                    unrecoverable!(stream, "double quotes are required for VAR=value pairs")
+                } else {
+                    make(vec![name])
+                }
             }
         };
 
@@ -740,7 +762,7 @@ impl Parse for defaults::SettingsModifier {
         // Parse a text parameter
         let text_item = |stream: &mut CharStream| {
             if stream.eat_char('"') {
-                let QuotedText(text) = expect_nonterminal(stream)?;
+                let QuotedStringParameter(text) = expect_nonterminal(stream)?;
                 expect_syntax('"', stream)?;
                 make(text)
             } else {
