@@ -1,7 +1,10 @@
 use super::Sudoers;
 
 use super::Judgement;
-use crate::common::{SudoPath, HARDENED_ENUM_VALUE_0, HARDENED_ENUM_VALUE_1};
+use crate::common::{
+    SudoPath, HARDENED_ENUM_VALUE_0, HARDENED_ENUM_VALUE_1, HARDENED_ENUM_VALUE_2,
+};
+use crate::sudoers::ast::Tag;
 use crate::system::{time::Duration, Hostname, User};
 /// Data types and traits that represent what the "terms and conditions" are after a succesful
 /// permission check.
@@ -29,14 +32,16 @@ pub struct Authentication {
 }
 
 impl super::Settings {
-    fn to_auth(&self) -> Authentication {
+    pub(super) fn to_auth(&self, tag: &Tag) -> Authentication {
         Authentication {
-            must_authenticate: true,
+            must_authenticate: tag.needs_passwd(),
             allowed_attempts: self.passwd_tries().try_into().unwrap(),
             prior_validity: Duration::seconds(self.timestamp_timeout()),
             pwfeedback: self.pwfeedback(),
             credential: if self.rootpw() {
                 AuthenticatingUser::Root
+            } else if self.targetpw() {
+                AuthenticatingUser::TargetUser
             } else {
                 AuthenticatingUser::InvokingUser
             },
@@ -68,16 +73,14 @@ pub enum DirChange<'a> {
 pub enum AuthenticatingUser {
     InvokingUser = HARDENED_ENUM_VALUE_0,
     Root = HARDENED_ENUM_VALUE_1,
+    TargetUser = HARDENED_ENUM_VALUE_2,
 }
 
 impl Judgement {
     pub fn authorization(&self) -> Authorization<Restrictions> {
         if let Some(tag) = &self.flags {
             Authorization::Allowed(
-                Authentication {
-                    must_authenticate: tag.needs_passwd(),
-                    ..self.settings.to_auth()
-                },
+                self.settings.to_auth(tag),
                 Restrictions {
                     use_pty: self.settings.use_pty(),
                     trust_environment: tag.allows_setenv(),
@@ -107,10 +110,6 @@ impl Sudoers {
         self.specify_host_user_runas(on_host, current_user, target_user);
         self.settings.secure_path()
     }
-
-    pub fn validate_authorization(&self) -> Authorization<()> {
-        Authorization::Allowed(self.settings.to_auth(), ())
-    }
 }
 
 #[cfg(test)]
