@@ -23,6 +23,11 @@ impl Token for Username {
     fn accept_1st(c: char) -> bool {
         c != '_' && Self::accept(c)
     }
+
+    const ALLOW_ESCAPE: bool = true;
+    fn escaped(c: char) -> bool {
+        matches!(c, '\\' | '"' | ',' | ':' | '=' | '!' | '(' | ')')
+    }
 }
 
 impl Many for Username {}
@@ -275,13 +280,14 @@ impl Token for EnvVar {
     }
 }
 
-pub struct QuotedText(pub String);
+/// A token with a very liberal inner tokenizer; compare StringParameter below
+pub struct QuotedStringParameter(pub String);
 
-impl Token for QuotedText {
+impl Token for QuotedStringParameter {
     const MAX_LEN: usize = 1024;
 
     fn construct(s: String) -> Result<Self, String> {
-        Ok(QuotedText(s))
+        Ok(Self(s))
     }
 
     fn accept(c: char) -> bool {
@@ -294,15 +300,17 @@ impl Token for QuotedText {
     }
 }
 
+/// Similar to QuotedStringParameter but treats backslashes differently
+/// Compare IncludePath below.
 // `@include "some/path"`
 //           ^^^^^^^^^^^
-pub struct QuotedInclude(pub String);
+pub struct QuotedIncludePath(pub String);
 
-impl Token for QuotedInclude {
+impl Token for QuotedIncludePath {
     const MAX_LEN: usize = 1024;
 
     fn construct(s: String) -> Result<Self, String> {
-        Ok(QuotedInclude(s))
+        Ok(Self(s))
     }
 
     fn accept(c: char) -> bool {
@@ -338,7 +346,7 @@ impl Token for IncludePath {
 pub struct StringParameter(pub String);
 
 impl Token for StringParameter {
-    const MAX_LEN: usize = QuotedText::MAX_LEN;
+    const MAX_LEN: usize = QuotedStringParameter::MAX_LEN;
 
     fn construct(s: String) -> Result<Self, String> {
         Ok(StringParameter(s))
@@ -391,3 +399,26 @@ impl Token for ChDir {
         matches!(c, '\\' | '"' | ' ')
     }
 }
+
+/// Some tokens that support escape characters also support being surrounded by quotes to avoid escaping directly.
+pub struct Unquoted<T>(pub String, pub std::marker::PhantomData<T>);
+
+impl<T: Token> Token for Unquoted<T> {
+    const MAX_LEN: usize = 1024;
+
+    fn construct(text: String) -> Result<Self, String> {
+        let mut quoted = String::new();
+        for ch in text.chars() {
+            if T::escaped(ch) {
+                quoted.push('\\');
+            }
+            quoted.push(ch);
+        }
+
+        Ok(Self(quoted, std::marker::PhantomData))
+    }
+
+    fn accept(c: char) -> bool {
+        c != '"' && !c.is_control()
+    }
+}
