@@ -13,6 +13,8 @@ impl Chown for File {
     fn chown(&self, owner: UserId, group: GroupId) -> io::Result<()> {
         let fd = self.as_raw_fd();
 
-        cerr(unsafe { libc::fchown(fd, owner, group) }).map(|_| ())
+        // SAFETY: `fchown` is passed a proper file descriptor; and even if the user/group id
+        // is invalid, it will not cause UB.
+        cerr(unsafe { libc::fchown(fd, owner.inner(), group.inner()) }).map(|_| ())
     }
 }
